from api.route import Route
from api_graphql.abc.mutation_abc import MutationABC
from api_graphql.field.mutation_field_builder import MutationFieldBuilder
from api_graphql.input.user_space_create_input import UserSpaceCreateInput
from api_graphql.input.user_space_update_input import UserSpaceUpdateInput
from core.logger import APILogger
from core.string import first_to_lower
from data.schemas.administration.user_dao import userDao
from data.schemas.public.user_space import UserSpace
from data.schemas.public.user_space_dao import userSpaceDao
from data.schemas.public.user_space_user import UserSpaceUser
from data.schemas.public.user_space_user_dao import userSpaceUserDao
from service.permission.permissions_enum import Permissions

logger = APILogger(__name__)


class UserSpaceMutation(MutationABC):
    def __init__(self):
        MutationABC.__init__(self, "UserSpace")

        self.field(
            MutationFieldBuilder("create")
            .with_resolver(self.resolve_create)
            .with_input(UserSpaceCreateInput)
            .with_change_broadcast(
                f"{first_to_lower(self.name.replace("Mutation", ""))}Change"
            )
            .with_require_any_permission([Permissions.user_spaces_create])
        )

        self.field(
            MutationFieldBuilder("update")
            .with_resolver(self.resolve_update)
            .with_input(UserSpaceUpdateInput)
            .with_change_broadcast(
                f"{first_to_lower(self.name.replace("Mutation", ""))}Change"
            )
            .with_require_any_permission([Permissions.user_spaces_update])
        )

        self.field(
            MutationFieldBuilder("delete")
            .with_resolver(self.resolve_delete)
            .with_change_broadcast(
                f"{first_to_lower(self.name.replace("Mutation", ""))}Change"
            )
            .with_require_any_permission([Permissions.user_spaces_delete])
        )

        self.field(
            MutationFieldBuilder("restore")
            .with_resolver(self.resolve_restore)
            .with_change_broadcast(
                f"{first_to_lower(self.name.replace("Mutation", ""))}Change"
            )
            .with_require_any_permission([Permissions.user_spaces_delete])
        )

    async def resolve_create(self, obj: UserSpaceCreateInput, *_):
        logger.debug(f"create user_space: {obj.__dict__}")

        already_exists = await userSpaceDao.find_by({UserSpace.name: obj.name})
        if len(already_exists) > 0:
            raise ValueError(f"UserSpace {obj.name} already exists")

        user_space = UserSpace(0, obj.name, (await Route.get_user()).id)
        gid = await userSpaceDao.create(user_space)

        await self._resolve_assignments(
            obj.get("users", []),
            user_space,
            UserSpaceUser.user_space_id,
            UserSpaceUser.user_id,
            userSpaceDao,
            userSpaceUserDao,
            UserSpaceUser,
            userDao,
        )

        return await userSpaceDao.get_by_id(gid)

    async def resolve_update(self, obj: UserSpaceUpdateInput, *_):
        logger.debug(f"update user_space: {input}")

        if await userSpaceDao.find_by_id(obj.id) is None:
            raise ValueError(f"UserSpace with id {obj.id} not found")

        user_space = await userSpaceDao.get_by_id(obj.id)
        if obj.name is not None:
            already_exists = await userSpaceDao.find_by(
                {UserSpace.name: obj.name, UserSpace.id: {"ne": obj.id}}
            )
            if len(already_exists) > 0:
                raise ValueError(f"UserSpace {obj.name} already exists")

            user_space.name = obj.name
            await userSpaceDao.update(user_space)

        await self._resolve_assignments(
            obj.get("users", []),
            user_space,
            UserSpaceUser.user_space_id,
            UserSpaceUser.user_id,
            userSpaceDao,
            userSpaceUserDao,
            UserSpaceUser,
            userDao,
        )

        return await userSpaceDao.get_by_id(obj.id)

    @staticmethod
    async def resolve_delete(*_, id: str):
        logger.debug(f"delete user_space: {id}")
        user_space = await userSpaceDao.get_by_id(id)
        await userSpaceDao.delete(user_space)
        return True

    @staticmethod
    async def resolve_restore(*_, id: str):
        logger.debug(f"restore user_space: {id}")
        user_space = await userSpaceDao.get_by_id(id)
        await userSpaceDao.restore(user_space)
        return True
